/*
 *
 * Copyright © 2007 Sun Microsystems, Inc. All rights reserved.
 * Use is subject to license terms.
 */
package com.sun.svg.component;

import org.w3c.dom.Document;
import org.w3c.dom.svg.SVGAnimationElement;
import org.w3c.dom.svg.SVGElement;
import org.w3c.dom.svg.SVGLocatableElement;
import org.w3c.dom.svg.SVGMatrix;
import org.w3c.dom.svg.SVGRect;
import org.w3c.dom.svg.SVGSVGElement;

/**
 * The convention for a scroll bar is that the background element has the 'bkg' suffix
 * (e.g., "myScrollBar.bkg") and the thumb element has the suffix 'thumb' 
 * (e.g., "myScrollBar.thumb").
 */
public class SVGHorizontalScrollBar {
    static final String BACKGROUND_SUFFIX = "bkg";
    static final String THUMB_SUFFIX = "thumb";
    
    /**
     * The scroll bar thumb
     */
    protected SVGLocatableElement thumb;
    
    /**
     * The thumb's initial transform.
     */
    protected SVGMatrix thumbTxf;
    
    /**
     * The min position of the thumb along the x-axis
     */
    protected float minTranslate;
    
    /**
     * The max position of the thumb along the x-axis
     */
    protected float maxTranslate;
    
    /**
     * The document's root svg element.
     */
    protected SVGSVGElement svg;
    
    /**
     * The scroll bar's id prefix, i.e., the prefix used for all of the 
     * scrollbar component ids.
     */
    protected String idPrefix;
    
    /**
     * The current scrollbar position.
     */
    protected float pos;
    
    /**
     * @param idPrefix the scrollBar id prefix.
     */
    public SVGHorizontalScrollBar(final String idPrefix) {
        if (idPrefix == null || "".equals(idPrefix)) {
            throw new IllegalArgumentException("SVGHorizontalScrollBar " +
                    "idPrefix should not be null or empty string");
        }
        
        this.idPrefix = idPrefix;
    }
    
    /**
     * Hooks this scroll bar with a new skin.
     *
     * @param doc - the new skin content.
     */
    public void hookSkin(final Document doc) {
        svg = (SVGSVGElement) doc.getDocumentElement();
        SVGLocatableElement bkg = (SVGLocatableElement) doc.getElementById(idPrefix + "." + BACKGROUND_SUFFIX);
        
        if (bkg == null) {
            throw new IllegalArgumentException("SVGHorizontalScrollBar : element with id " + idPrefix +
                    "." + BACKGROUND_SUFFIX + " does not exist in skin");
        }
        
        thumb = (SVGLocatableElement) doc.getElementById(idPrefix + "." + THUMB_SUFFIX);
        if (thumb == null) {
            throw new IllegalArgumentException("SVGHorizontalScrollBar : element with id " + idPrefix +
                    "." + THUMB_SUFFIX + " does not exist in skin");
        }
        
        SVGRect bkgBBox = bkg.getBBox();
        SVGRect thumbBBox = thumb.getBBox();
        
        thumbTxf = thumb.getMatrixTrait("transform");
        
        minTranslate = -thumbBBox.getX() + bkgBBox.getX();
        maxTranslate = -thumbBBox.getX() + bkgBBox.getX() + bkgBBox.getWidth() - thumbBBox.getWidth();
        
        setThumbPosition(pos);
        
    }
    
    /**
     * @param pos the desired thumb position in the [0, 1] interval. If the value is out of range, it
     * is clipped to the valid range.
     */
    public void setThumbPosition(float pos) {
        if (pos < 0) {
            pos = 0;
        } else if (pos > 1) {
            pos = 1;
        }
        
        float xTranslate = minTranslate + pos * (maxTranslate - minTranslate);
        SVGMatrix txf = svg.createSVGMatrixComponents(1, 0, 0, 1, 0, 0);
        txf.mMultiply(thumbTxf);
        txf.mTranslate(xTranslate, 0);
        thumb.setMatrixTrait("transform", txf);
        this.pos = pos;
    }

}

